#version 330 core


struct Light {
    vec3 position;  
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;

	float constant;
    float linear;
    float quadratic;
};
struct Material {
	float shininess;
    vec3 diffuse;
	vec3 ambient;
	vec3 specular;
    float opacity;	
}; 

in VS_OUT {
     vec2 TexCoord;
	 vec3 FragPos;
	 mat3 TBN;
	 vec3 Normal;

} fs_in;


out vec4 color;

uniform sampler2D texture_diffuse1;
uniform sampler2D texture_specular1;
uniform sampler2D texture_normal1;
uniform samplerCube depthMap;
uniform vec3 viewPos;
uniform Material material;
uniform Light light;  

uniform float far_plane;
uniform bool shadows;


float ShadowCalculation(vec3 fragPos)
{
    // Get vector between fragment position and light position
    vec3 fragToLight =   fragPos - light.position; 
    // Use the fragment to light vector to sample from the depth map    
    float closestDepth = texture(depthMap, fragToLight).r;
    // It is currently in linear range between [0,1]. Let's re-transform it back to original depth value
    closestDepth *= far_plane;
    // Now get current linear depth as the length between the fragment and light position
    float currentDepth = length(fragToLight);
    // Now test for shadows
    float bias = 0.05; // We use a much larger bias since depth is now in [near_plane, far_plane] range
    float shadow = currentDepth -  bias > closestDepth ? 1.0 : 0.0;



    return shadow;
}


void main()
{


 // Get diffuse color
    vec3 objColor = texture(texture_diffuse1, fs_in.TexCoord).rgb;
	
	// distance to light
	float distance    = length(light.position - fs_in.FragPos);
	// Light gets weaker when further away, and stronger when near
	float attenuation = 1.0f / (light.constant + light.linear * distance + light.quadratic * (distance * distance));    


  	// Ambient
	vec3 ambient = light.ambient * objColor;

    
    vec3 norm  = texture(texture_normal1, fs_in.TexCoord).rgb;
	 norm = normalize(norm * 2.0 - 1.0);   
	 norm = normalize(fs_in.TBN * norm); 
	 if(shadows != true)
	 norm = normalize(fs_in.Normal);
	

    vec3 lightDir = normalize(light.position - fs_in.FragPos);
    float diff = max(dot(norm, lightDir), 0.0);
    vec3 diffuse = light.diffuse * diff  * objColor ;  
    
    // Specular

    vec3 viewDir = normalize(viewPos - fs_in.FragPos);
	vec3 halfwayDir = normalize(lightDir + viewDir);
     float spec = pow(max(dot(norm, halfwayDir), 0.0), material.shininess);
    vec3 specular = light.specular * spec  * vec3(texture(texture_specular1, fs_in.TexCoord));  
	 // Calculate shadow

	float shadow = shadows ? ShadowCalculation(fs_in.FragPos) : 0.0;   
	

    vec3 result = (ambient + (1.0 - shadow) * (diffuse + specular)) * attenuation;
    color = vec4(result, 1.0f) ;
	
	

	}